Option Strict Off
Option Explicit On

Imports System.Runtime.InteropServices
Imports System.Reflection
Imports System.IO

Module CommonModule
    Public H As Double
    Public XEND As Double
    Public K As Integer
    Public Const N As Integer = 3
    Private Outfilename As String = CurDir() + "\d02ejfe_out.txt"
    Public Outstream As New FileStream(Outfilename, FileMode.Create, FileAccess.Write)
    Public OutfileWriter As New StreamWriter(Outstream)
End Module

Module String_Stuff
    Public Const NewLine As String = Chr(10) & Chr(13)
End Module

Module Module1

    Sub Main()
        Dim J As Integer
        Dim TOL As Double
        Dim X As Double
        Dim Y(N - 1) As Double
        Dim IFAIL As Integer
        Const IW As Integer = (12 + N) * N + 50
        Dim W(IW) As Double

        Console.WriteLine("D02EJF Example Program Results")
        XEND = 10.0
        Console.WriteLine("")
        Console.WriteLine("Case 1: calculating Jacobian internally,")
        Console.WriteLine(" intermediate output, root-finding")
        For J = 3 To 4
            TOL = 10 ^ (-J)
            Console.WriteLine("")
            Console.WriteLine(" Calculation with TOL = " & Format(TOL, "E"))
            X = 0
            Y(0) = 1
            Y(1) = 0
            Y(2) = 0
            K = 4
            H = (XEND - X) / (K + 1)
            Console.WriteLine("     X         Y(1)         Y(2)         Y(3)")
            IFAIL = 1
            D02EJF(X, XEND, N, Y, AddressOf FCN, AddressOf D02EJY, _
                   TOL, "Default", 8, _
                   AddressOf Out, AddressOf G, W, IW, IFAIL)
            Console.WriteLine("   Root of Y(1)-0.9 at " & Format(X, "E"))
            Console.WriteLine("   Solution is " & _
                              Format(Y(0), "E") & " " & _
                              Format(Y(1), "E") & " " & _
                              Format(Y(2), "E"))
            If (TOL < 0) Then
                Console.WriteLine("  Range too short for TOL")
            End If
        Next
        Console.WriteLine("")
        Console.WriteLine("")
        Console.WriteLine("Case 2: calculating Jacobian by PEDERV,")
        Console.WriteLine("intermediate output, root-finding")
        For J = 3 To 4
            TOL = 10 ^ (-J)
            Console.WriteLine("")
            Console.WriteLine(" Calculation with TOL = " & Format(TOL, "E"))
            X = 0
            Y(0) = 1
            Y(1) = 0
            Y(2) = 0
            K = 4
            H = (XEND - X) / (K + 1)
            Console.WriteLine("     X         Y(1)         Y(2)         Y(3)")
            IFAIL = 1
            D02EJF(X, XEND, N, Y, AddressOf FCN, AddressOf PEDERV, _
                   TOL, "Default", 8, _
                   AddressOf Out, AddressOf G, W, IW, IFAIL)
            Console.WriteLine("   Root of Y(1)-0.9 at " & Format(X, "E"))
            Console.WriteLine("   Solution is " & _
                              Format(Y(0), "E") & " " & _
                              Format(Y(1), "E") & " " & _
                              Format(Y(2), "E"))
            If (TOL < 0) Then
                Console.WriteLine("  Range too short for TOL")
            End If
        Next
        Console.WriteLine("")
        Console.WriteLine("")
        Console.WriteLine("Case 3: calculating Jacobian internally,")
        Console.WriteLine(" no intermediate output, root-finding")
        For J = 3 To 4
            TOL = 10 ^ (-J)
            Console.WriteLine("")
            Console.WriteLine(" Calculation with TOL = " & Format(TOL, "E"))
            X = 0
            Y(0) = 1
            Y(1) = 0
            Y(2) = 0
            IFAIL = 1
            D02EJF(X, XEND, N, Y, AddressOf FCN, AddressOf D02EJY, _
                   TOL, "Default", 8, _
                   AddressOf D02EJX, AddressOf G, W, IW, IFAIL)
            Console.WriteLine("   Root of Y(1)-0.9 at " & Format(X, "E"))
            Console.WriteLine("   Solution is " & _
                              Format(Y(0), "E") & " " & _
                              Format(Y(1), "E") & " " & _
                              Format(Y(2), "E"))
            If (TOL < 0) Then
                Console.WriteLine("  Range too short for TOL")
            End If
        Next
        Console.WriteLine("")
        Console.WriteLine("")
        Console.WriteLine("Case 4: calculating Jacobian internally,")
        Console.WriteLine(" intermediate output, no root-finding")
        For J = 3 To 4
            TOL = 10 ^ (-J)
            Console.WriteLine("")
            Console.WriteLine(" Calculation with TOL = " & Format(TOL, "E"))
            X = 0
            Y(0) = 1
            Y(1) = 0
            Y(2) = 0
            K = 4
            H = (XEND - X) / (K + 1)
            Console.WriteLine("     X         Y(1)         Y(2)         Y(3)")
            IFAIL = 1
            D02EJF(X, XEND, N, Y, AddressOf FCN, AddressOf D02EJY, _
                   TOL, "Default", 8, _
                   AddressOf Out, AddressOf D02EJW, W, IW, IFAIL)
            If (TOL < 0) Then
                Console.WriteLine("  Range too short for TOL")
            End If
        Next
        Console.WriteLine("")
        Console.WriteLine("")
        Console.WriteLine("Case 5: calculating Jacobian internally,")
        Console.WriteLine(" no intermediate output, no root-finding (integrate to XEND)")
        For J = 3 To 4
            TOL = 10 ^ (-J)
            Console.WriteLine("")
            Console.WriteLine(" Calculation with TOL = " & Format(TOL, "E"))
            X = 0
            Y(0) = 1
            Y(1) = 0
            Y(2) = 0
            Console.WriteLine("     X         Y(1)         Y(2)         Y(3)")
            IFAIL = 1
            Console.WriteLine(Format(X, "E") & " " & _
                              Format(Y(0), "E") & " " & _
                              Format(Y(1), "E") & " " & _
                              Format(Y(2), "E"))
            D02EJF(X, XEND, N, Y, AddressOf FCN, AddressOf D02EJY, _
                   TOL, "Default", 8, _
                   AddressOf D02EJX, AddressOf D02EJW, W, IW, IFAIL)
            Console.WriteLine(Format(X, "E") & " " & _
                  Format(Y(0), "E") & " " & _
                  Format(Y(1), "E") & " " & _
                  Format(Y(2), "E"))
            If (TOL < 0) Then
                Console.WriteLine("  Range too short for TOL")
            End If
        Next
        OutfileWriter.Close()
    End Sub

    Sub FCN(ByRef T As Double, ByVal Yptr As IntPtr, ByVal Fptr As IntPtr)
        Dim Y(N - 1) As Double
        Dim F(N - 1) As Double
        Marshal.Copy(Yptr, Y, 0, N)

        F(0) = -0.04 * Y(0) + 10000 * Y(1) * Y(2)
        F(1) = 0.04 * Y(0) - 10000 * Y(1) * Y(2) - 30000000.0 * Y(1) * Y(1)
        F(2) = 30000000.0 * Y(1) * Y(1)
        Marshal.Copy(F, 0, Fptr, N)
    End Sub

    Sub PEDERV(ByRef X As Double, ByVal Yptr As IntPtr, _
            ByVal PW As IntPtr)
        ' PW transposed, treated as one-dimensional
        ' You could instead use a local 2D array and CopyMemory
        ' the result to PW.
        Dim Yloc(N) As Double
        Dim PWloc(N * N) As Double
        Marshal.Copy(Yptr, Yloc, 0, N)
        PWloc(0) = -0.04
        PWloc(3) = 10000 * Yloc(2)
        PWloc(6) = 10000 * Yloc(1)
        PWloc(1) = 0.04
        PWloc(4) = -10000 * Yloc(2) - 60000000.0 * Yloc(1)
        PWloc(7) = -10000.0 * Yloc(1)
        PWloc(2) = 0.0
        PWloc(5) = 60000000.0 * Yloc(1)
        PWloc(8) = 0.0

        Marshal.Copy(PWloc, 0, PW, N * N)
    End Sub

    Function G(ByRef T As Double, ByVal Yptr As IntPtr) As Double
        Dim Y(N) As Double
        Marshal.Copy(Yptr, Y, 0, N)
        G = Y(0) - 0.9
        Exit Function
    End Function

    Sub Out(ByRef X As Double, ByVal Yptr As IntPtr)
        Dim I As Integer
        Dim Msg As String
        Dim Y(N - 1) As Double
        Marshal.Copy(Yptr, Y, 0, N)
        Msg = Format(X, "E")
        For I = 0 To (N - 1)
            Msg = Msg & " " & Format(Y(I), "E")
        Next
        OutfileWriter.WriteLine(Msg)
        OutfileWriter.Flush()
        REM Msg = Msg & NewLine
        Console.WriteLine(Msg)
        X = XEND - K * H
        REM Console.WriteLine("XEND=" & Format(XEND, "E"))
        REM Console.WriteLine("K=" & Format(K, "D"))
        REM Console.WriteLine("H=" & Format(H, "E"))
        REM Console.WriteLine("X=" & Format(X, "E"))
        K = K - 1
    End Sub

End Module
